"use strict";

    this.name    = "Docked_HUDs_data_sources";
 // this.name    = "data";
    this.version = "1.3";

/* ========================================================
			STILL TO-DO

 1. The flag, _repairsNeeded, is surprisingly frustrating.
    It encourages me to check the F3 screen, where I learn
    the repairs I need are not available on this low-tech
    station. What's actually needed is a _repairsAvailable
    flag.

 3. IMPORTANT: efficient versions of the guiScreenChanged
    and viewDirectionChanged functions that don't just set
    all eight labels every time.
=========================================================== */

    this._useInFlight = false; // Always false in published version.

    this._checkForDamage = function () {
        "use strict";
        var self = player.ship;
        for (var i = 0; i < self.equipment.length; i++) {
            if (self.equipment[i].isVisible &&
                self.equipmentStatus(self.equipment[i]) === "EQUIPMENT_DAMAGED") {
                this._repairsNeeded = true;
                return true;
            }
        }
        this._repairsNeeded = false;
        return false;
    }

/* ========================================================
			MONITORING GOINGS ON

    _statusChanged, _repairsNeeded & _cashLow determine
    whether F5 is described as "Status" or "STATUS".
    So much effort for such a subtle effect. Sigh.
=========================================================== */

    this._repairsNeeded = null;
    this._statusChanged = false;
    this._cashLow = false;
    this._legalStatus = null;
    this._rank = null;
    this._cashState = null;

/* ========================================================
			BLANK OR SET ALL LABELS

    Call gameResumed if we need to re-set all eight labels.
=========================================================== */

    this._blankLabels = function () {
        "use strict";
        var setDial = player.ship.setCustomHUDDial;
        for (var i = 1; i < 9; i++) {
            setDial("functionKey" + i, "");
        }
    }

    this.gamePaused = function () {
        "use strict";
        this._blankLabels();
        var self = player.ship;
        if (this._useInFlight && !self.docked) {
            self.setCustomHUDDial("functionKey2", "OPTIONS");
        }
    }

    this.gameResumed = function () {
        "use strict";
        this._selectF1Text();
        this._selectF2Text();
        this._selectF3Text();
        this._selectF4Text();
        this._selectF5Text();
        this._selectF6Text();
        this._selectF7Text();
        this._selectF8Text();
    }

/* ========================================================
			LABELS 1-4

    These buttons are complicated by the fact that their 
    function differs when docked and in flight.
    Buttons 5, 6, 7 & 8 should be simpler.
=========================================================== */

    this._selectF1Text = function () {
        "use strict";
        var self = player.ship;
        if (self.docked) {
            self.setCustomHUDDial("functionKey1", "Launch");
            return;
        } else {
            if (this._useInFlight && self.viewDirection !== "VIEW_FORWARD") {
                self.setCustomHUDDial("functionKey1", "FORWARD");
                return;
            } else {
                self.setCustomHUDDial("functionKey1", "");
                return false;
            }
        }
    }

    this._selectF2Text = function () {
        "use strict";
        var self = player.ship;
        if (guiScreen === "GUI_SCREEN_OPTIONS") {
            self.setCustomHUDDial("functionKey2", "");
            return;
        } else if (self.docked) {
            self.setCustomHUDDial("functionKey2", "Sleep");
            return;
        } else {
            if (this._useInFlight && self.viewDirection !== "VIEW_AFT") {
                self.setCustomHUDDial("functionKey2", "AFT");
                return;
            } else {
                self.setCustomHUDDial("functionKey2", "");
                return false;
            }
        }
    }

    this._selectF3Text = function () {
        "use strict";
        var self = player.ship;
        if (self.docked) {
            if (guiScreen === "GUI_SCREEN_EQUIP_SHIP") {
                self.setCustomHUDDial("functionKey3", "Shipyard");
                return;
            } else if (self.fuel < 7) {
                self.setCustomHUDDial("functionKey3", "RE-FUEL");
                return;
            } else if (this._checkForDamage()) {
                self.setCustomHUDDial("functionKey3", "REPAIRS");
                return;
            } else if (self.missileCapacity > self.missiles.length) {
                self.setCustomHUDDial("functionKey3", "RE-ARM");
                return;
            } else {
                self.setCustomHUDDial("functionKey3", "Equipment");
                return;
            }
        } else {
            if (this._useInFlight && self.viewDirection !== "VIEW_PORT") {
                self.setCustomHUDDial("functionKey3", "PORT");
                return;
            } else {
                self.setCustomHUDDial("functionKey3", "");
                return false;
            }
        }
    }

    this._selectF4Text = function () {
        "use strict";
        var self = player.ship;
        if (self.docked) {
            if (guiScreen === "GUI_SCREEN_INTERFACES") {
                self.setCustomHUDDial("functionKey4", "");
                return;
            }
            let es = worldScripts.EmailSystem;
            if (es && es.$totalUnreadItems() > 0) {
                self.setCustomHUDDial("functionKey4", "MAIL");
                return;
            } else {
                self.setCustomHUDDial("functionKey4", "Services");
                return;
            }
        } else {
            if (this._useInFlight && self.viewDirection !== "VIEW_STARBOARD") {
                self.setCustomHUDDial("functionKey4", "STARBOARD");
                return;
            } else {
                self.setCustomHUDDial("functionKey4", "");
                return false;
            }
        }
    }

/* ========================================================
			F5 EVENT LISTENERS

    _checkForLowCash is called on every playerBought event
    to alert the player if he is getting scandalously short
    of money.

    alertConditionChanged is used to monitor the player's
    legal and killer-elite statuses.
=========================================================== */

    this._checkForLowCash = function () {
        "use strict";
        if (!this._cashLow) {
            var cash = player.credits.toString().length;
            if (cash < this._cashState) {
                this._cashLow = true;
                this._cashState = cash;
                this._selectF5Text();
            } else {
                this._cashLow = false;
            }
        }
    }

    this.playerBoughtCargo = function () {
        "use strict";
        this._checkForLowCash();
    }

    this.playerBoughtEquipment = function () {
        "use strict";
        this._checkForDamage();
        this._checkForLowCash();
        this._selectF3Text();
    }

    this.playerBoughtNewShip = function () {
        "use strict";
        this._checkForLowCash();
    }

    this.alertConditionChanged = function () {
        "use strict";
        if (!this._statusChanged) {
            if (this._legalStatus !== player.legalStatus ||
                this._rank !== player.rank) {
                this._statusChanged = true;
                this._legalStatus = player.legalStatus;
                this._rank = player.rank;
            }
        }
    }

/* ========================================================
			LABELS 5-8

    These buttons are simpler because the text will be the
    same whether docked or in flight. So, the condition tests
    if (self.docked) or if (this._useInFlight) are able to
    be combined.
=========================================================== */

    this._selectF5Text = function () {
        "use strict";
        var self = player.ship;
        if (self.docked || this._useInFlight) {
            if (guiScreen === "GUI_SCREEN_STATUS") {
                self.setCustomHUDDial("functionKey5", "Manifest");
                this._statusChanged = false;
                return;
            } else if (this._statusChanged || this._repairsNeeded) {
                self.setCustomHUDDial("functionKey5", "STATUS");
                return;
            } else if (this._cashLow) {
                self.setCustomHUDDial("functionKey5", "CASH");
                return;
            } else {
                self.setCustomHUDDial("functionKey5", "Status");
                return;
            }
        } else {
            self.setCustomHUDDial("functionKey5", "");
            return false;
        }
    }

    this._selectF6Text = function () {
        "use strict";
        var self = player.ship;
        if (self.docked || this._useInFlight) {
            if (guiScreen === "GUI_SCREEN_SHORT_RANGE_CHART") {
                self.setCustomHUDDial("functionKey6", "Route Planner");
                return;
            } else if (guiScreen === "GUI_SCREEN_LONG_RANGE_CHART") {
                self.setCustomHUDDial("functionKey6", "Shorter Range");
                return;
            } else if (self.targetSystem === system.ID) {
                self.setCustomHUDDial("functionKey6", "DESTINATION");
                return;
            } else {
                self.setCustomHUDDial("functionKey6", "Navigation");
                return;
            }
        } else {
            self.setCustomHUDDial("functionKey6", "");
            return false;
        }
    }

    this._selectF7Text = function () {
        "use strict";
        var self = player.ship;
        if (self.docked || this._useInFlight) {
            if (guiScreen === "GUI_SCREEN_SYSTEM_DATA") {
                self.setCustomHUDDial("functionKey7", "");
                return;
            } else {
                self.setCustomHUDDial("functionKey7", "Database");
                return;
            }
        } else {
            self.setCustomHUDDial("functionKey7", "");
            return false;
        }
    }

    this._selectF8Text = function () {
        "use strict";
        var self = player.ship;
        if (self.docked || this._useInFlight) {
            if (guiScreen === "GUI_SCREEN_MARKET") {
                self.setCustomHUDDial("functionKey8", "Commodity");
                return;
            } else if (guiScreen === "GUI_SCREEN_MARKETINFO") {
                self.setCustomHUDDial("functionKey8", "All Commodities");
                return;
            } else {
                self.setCustomHUDDial("functionKey8", "Market");
                return;
            }
        } else {
            self.setCustomHUDDial("functionKey8", "");
            return false;
        }
    }

/* ========================================================
			MAIN EVENT LISTENERS

    Sigh. So tempting to just use the easy opton.
=========================================================== */

    this.viewDirectionChanged = function (to, from) {
        "use strict";
        if (this._useInFlight) {
            this._selectF1Text();
            this._selectF2Text();
            this._selectF3Text();
            this._selectF4Text();
            if (to === "VIEW_GUI_SCREEN") {
                this._selectF5Text();
                this._selectF6Text();
                this._selectF7Text();
                this._selectF8Text();
            }
        }
    }

    this.guiScreenChanged = function (to, from) {
        "use strict";
        if (player.ship.docked || this._useInFlight) {
            if (from === "GUI_SCREEN_MAIN") {
                this.gameResumed();
            } else {
                if (to === "GUI_SCREEN_STATUS" ||
                    from === "GUI_SCREEN_STATUS" ||
                    to === "GUI_SCREEN_MANIFEST" ||
                    from === "GUI_SCREEN_MANIFEST") {
                    this._selectF5Text();
                }
                if (to === "GUI_SCREEN_MARKET" ||
                    from === "GUI_SCREEN_MARKET" ||
                    to === "GUI_SCREEN_MARKETINFO" ||
                    from === "GUI_SCREEN_MARKETINFO") {
                    this._selectF8Text();
                }
                if (to === "GUI_SCREEN_EQUIP_SHIP" ||
                    from === "GUI_SCREEN_EQUIP_SHIP" ||
                    to === "GUI_SCREEN_SHIPYARD" ||
                    from === "GUI_SCREEN_SHIPYARD") {
                    this._selectF3Text();
                }
                if (to === "GUI_SCREEN_SHORT_RANGE_CHART" ||
                    from === "GUI_SCREEN_SHORT_RANGE_CHART" ||
                    to === "GUI_SCREEN_LONG_RANGE_CHART" ||
                    from === "GUI_SCREEN_LONG_RANGE_CHART") {
                    this._selectF6Text();
                }
                if (to === "GUI_SCREEN_INTERFACES" ||
                    from === "GUI_SCREEN_INTERFACES") {
                    this._selectF4Text();
                }
                if (to === "GUI_SCREEN_SYSTEM_DATA" ||
                    from === "GUI_SCREEN_SYSTEM_DATA") {
                    this._selectF7Text();
                }
                if (to === "GUI_SCREEN_OPTIONS" ||
                    from === "GUI_SCREEN_OPTIONS") {
                    this._selectF2Text();
                }
            }
        }
    }

/* ========================================================
			EVENT LISTENERS, Cont.d
=========================================================== */

    this.startUpComplete = function () {
        "use strict";
        this._statusChanged = false;
        this._legalStatus = player.legalStatus;
        this._rank = player.rank;
        this._cashState = player.credits.toString().length;
    }

    this.shipWillLaunchFromStation = function () {
        "use strict";
        this._blankLabels();
    }

    this.shipLaunchedFromStation = function () {
        "use strict";
        if (this._useInFlight) {
            this.gameResumed();
        }
    }

/* ========================================================
			THE END
=========================================================== */